//*************************************************************
//**		Titre: Reconnaissance Vocale                 **
//**		Noms: DavPichFabFab                          **
//**		Date: 22/01/2004                             **
//*************************************************************

#include <90s8535.h>
#include <delay.h>

// Seuils
#define  seuilclap_haut		700       // 3.4V
#define  seuilclap_bas		300       // 1,5V
#define  seuilinterclap_bas	350       // 1,7V
#define  Temps_clap_min		100       // 100ms
#define  Temps_clap_max		350       // 350ms
#define  Temps_interclap_min	100       // 100ms
#define  Temps_interclap_max	500       // 500ms

// Ordres
#define fermer 0
#define ouvrir 1

// Commandes du portail
#define Ouverture PORTB.0
#define Fermeture PORTB.1

//sous programmes
unsigned char clap(void);
unsigned char interclap(void);

// Variables globales
unsigned char Nb_Clap_interclap_Valide=0; // Compteur de clap suivi de interclap valides

// Timer 1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_ovf_isr(void)
{
// Reinitialize Timer 1 value
TCCR1B=0x00; // Dsactivation du timer 1
TCNT1H=0xE1; // Rechargement  la valeur initiale
TCNT1L=0x7C; // de 2s avant interruption
Nb_Clap_interclap_Valide = 0;

}
                   
// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input;
// Start the AD conversion
ADCSR|=0x40;
// Wait for the AD conversion to complete
while ((ADCSR & 0x10)==0);
ADCSR|=0x10;
return ADCW;
}

// Programme principal
void main(void)
{
//Variables locales		             type        Commentaires                         
unsigned char Clap_Valide=0;      	  // Boolen 	V = 1 si clap valide
unsigned char Interclap_Valide=0;      	  // Boolen 	W = 1 si interclap valide
unsigned char ordre_precedent = fermer;   // Boolen    ralise une bascule entre ouverture 
					  //            et fermeture  

// Configuration du PortA en entre
DDRA=0x00;

// Configuration du PORTB en sortie					  
DDRB= 0x03; // bits 1 et 2 du PORTB en sortie

// ADC initialization
// ADC Clock frequency: 125,000 kHz
ADCSR=0x85;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 3,906 kHz
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0xE1;
TCNT1L=0x7C;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x04;

#asm("sei");	     
					  
while (1)
 {      				//debut prog princ
 
 do             	       		// Attente clap valide
   Clap_Valide = clap();
 while (Clap_Valide == 0);
 
 Clap_Valide = 0;
 Interclap_Valide = interclap();	// Mesure interclap	
 
 if (Interclap_Valide==1)      		// Si interclap valide
   {
    Nb_Clap_interclap_Valide = Nb_Clap_interclap_Valide + 1;    // Compter le nombre de claps suivis d'interclaps
    if (Nb_Clap_interclap_Valide == 1) 
    	{
	 TCCR1B=0x05; // Activation du timer 1, RAZ Nb_Clap_interclap_Valide si les suivants arrivent en plus de 2s
	}
    if ((Nb_Clap_interclap_Valide==2)&&(ordre_precedent == ouvrir))   // Si gal  3 => ordre de fermeture pendant 1s
    	{
         TCCR1B=0x00; // Dsactivation du timer 1
         TCNT1H=0xE1; // Rechargement  la valeur initiale
	 TCNT1L=0x7C; // de 2s avant interruption

    	 Fermeture=1;
    	 delay_ms(1000);
    	 Nb_Clap_interclap_Valide=0;
    	 Fermeture=0;
    	 ordre_precedent = fermer;
    	}
    	else if ((Nb_Clap_interclap_Valide==1)&& (ordre_precedent == fermer)) // Si gal  2 => ordre d'ouverture pendant 1s
    	{
	 TCCR1B=0x00; // Dsactivation du timer 1
         TCNT1H=0xE1; // Rechargement  la valeur initiale
	 TCNT1L=0x7C; // de 2s avant interruption

    	 Ouverture=1;
         delay_ms(1000);
    	 Nb_Clap_interclap_Valide=0;
    	 Ouverture=0;
    	 ordre_precedent = ouvrir;

    	}            // Sinon (1 seul clap) => attente clap suivant
    }
 else 
   {
     Nb_Clap_interclap_Valide = 0;
     Interclap_Valide = 0;
   }

 }// Fin du while

}//Fin du main


unsigned char clap(void)
{
unsigned int signal, Temps_clap=0;

signal=read_adc(7); // dtection dbut clap
if (signal > seuilclap_haut)
{ 
  do 
   {
    delay_ms(1);
    Temps_clap = Temps_clap + 1;
    signal=read_adc(7);
   }
  while ((signal > seuilclap_bas) && (Temps_clap < Temps_clap_max));
  if ((signal < seuilclap_bas) && (Temps_clap > Temps_clap_min) && (Temps_clap < Temps_clap_max)) return 1;
  		else return 0;
}
return 0;
}

unsigned char interclap(void)
{
unsigned int signal, Temps_interclap=0;

  do
   { 
    delay_ms(1);
    Temps_interclap = Temps_interclap + 1;
    signal=read_adc(7);
   }
  while ((signal < seuilinterclap_bas) && (Temps_interclap < Temps_interclap_max));
  if ((signal > seuilinterclap_bas) && (Temps_interclap > Temps_interclap_min) && (Temps_interclap < Temps_clap_max)) return 1;
  		else return 0;


}
